<?php
namespace Aelia\WC\TaxDisplayByCountry;
if(!defined('ABSPATH')) exit; // Exit if accessed directly

//define('SCRIPT_DEBUG', 1);
//error_reporting(E_ALL);

require_once('lib/classes/definitions/definitions.php');

use Aelia\WC\Aelia_Plugin;
use Aelia\WC\Aelia_SessionManager;
use Aelia\WC\IP2Location;
use Aelia\WC\TaxDisplayByCountry\Settings;
use Aelia\WC\TaxDisplayByCountry\Settings_Renderer;
use Aelia\WC\Messages;


/**
 * Tax Display by Country plugin.
 **/
class WC_Aelia_Tax_Display_By_Country extends Aelia_Plugin {
	public static $version = '1.7.12.151116';

	public static $plugin_slug = Definitions::PLUGIN_SLUG;
	public static $text_domain = Definitions::TEXT_DOMAIN;
	public static $plugin_name = 'Aelia Tax Display by Country for WooCommerce';

	// @var string The billing country currently active
	protected $billing_country = null;

	/**
	 * Customer's taxable address. Used for caching.
	 * @var array
	 * @since 1.7.7.150831
	 */
	protected $customer_taxable_address = null;

	public static function factory() {
		// Load Composer autoloader
		require_once(__DIR__ . '/vendor/autoload.php');

		$settings_controller = null;
		$messages_controller = null;
		// Example on how to initialise a settings controller and a messages controller
		$settings_page_renderer = new Settings_Renderer();
		$settings_controller = new Settings(Settings::SETTINGS_KEY,
																				self::$text_domain,
																				$settings_page_renderer);
		$messages_controller = new Messages();

		$plugin_instance = new self($settings_controller, $messages_controller);
		return $plugin_instance;
	}

	/**
	 * Constructor.
	 *
	 * @param \Aelia\WC\Settings settings_controller The controller that will handle
	 * the plugin settings.
	 * @param \Aelia\WC\Messages messages_controller The controller that will handle
	 * the messages produced by the plugin.
	 */
	public function __construct($settings_controller = null,
															$messages_controller = null) {
		// Load Composer autoloader
		require_once(__DIR__ . '/vendor/autoload.php');

		parent::__construct($settings_controller, $messages_controller);

		// The commented line below is needed for Codestyling Localization plugin to
		// understand what text domain is used by this plugin
		//load_plugin_textdomain(static::$text_domain, false, $this->path('languages') . '/');
	}

	/**
	 * Performs operation when woocommerce has been loaded.
	 */
	public function woocommerce_loaded() {
		parent::woocommerce_loaded();

		// Ensure that WooCommerce session is initialised
		self::init_woocommerce_session();

		// Update customer's country, to ensure that the correct taxes are applied
		$this->set_customer_country($this->get_billing_country(), isset($_POST[Definitions::ARG_AELIA_BILLING_COUNTRY]));
	}

	/**
	 * Determines if user has selected a billing country using the widget.
	 *
	 * @return bool
	 */
	protected function billing_country_selected() {
		return isset($_POST[Definitions::ARG_AELIA_BILLING_COUNTRY]);
	}

	/**
	 * Determines on what criteria to calculate the tax. Code copied from
	 * WC_Customer::get_taxable_address().
	 *
	 * @return string The criteria to calculate taxes.
	 *
	 * @since 1.6.1.141012
	 * @see WC_Customer::get_taxable_address()
	 */
	protected function tax_based_on() {
		$tax_based_on = get_option('woocommerce_tax_based_on');

		// Check shipping method at this point to see if we need special handling
		if(apply_filters('woocommerce_apply_base_tax_for_local_pickup', true) == true &&
			 WC()->cart->needs_shipping() &&
			 sizeof(array_intersect(WC()->session->get('chosen_shipping_methods', array(get_option('woocommerce_default_shipping_method'))), apply_filters('woocommerce_local_pickup_methods', array('local_pickup')))) > 0) {
			$tax_based_on = 'base';
		}

		return $tax_based_on;
	}

	/**
	 * Sets the billing and shipping country on the user object.
	 *
	 * @param string country A country code.
	 */
	protected function set_customer_country($country, $set_shipping_country = false) {
		$woocommerce = $this->wc();
		if(isset($woocommerce->customer)) {
			$woocommerce->customer->set_location($country, '');

			$shipping_country = trim($woocommerce->customer->get_shipping_country());
			if($set_shipping_country || empty($shipping_country)) {
				$woocommerce->customer->set_shipping_location($country);
			}
			// Trick WooCommerce into thinking that shipping has been calculated,
			// so that the appropriate tax rate is applied throughout the shop
			$woocommerce->customer->calculated_shipping(true);
		}
	}

	/**
	 * Sets the hooks required by the plugin.
	 */
	protected function set_hooks() {
		parent::set_hooks();

		if(!is_admin() || self::doing_ajax()) {
			// Add hooks to alter the tax display flag depending on user's country
			add_filter('option_woocommerce_tax_display_shop', array($this, 'option_woocommerce_tax_display_shop'));
			add_filter('option_woocommerce_tax_display_cart', array($this, 'option_woocommerce_tax_display_cart'));
			add_filter('woocommerce_get_price_suffix', array($this, 'woocommerce_get_price_suffix'), 10, 2);
			add_filter('woocommerce_countries_ex_tax_or_vat', array($this, 'woocommerce_countries_ex_tax_or_vat'), 10, 1);
			add_filter('woocommerce_countries_inc_tax_or_vat', array($this, 'woocommerce_countries_inc_tax_or_vat'), 10, 1);
			add_filter('woocommerce_check_cart_items', array($this, 'woocommerce_check_cart_items'), 10);

			// Add filters for "fixed product prices" feature
			add_action('woocommerce_before_calculate_totals', array($this, 'woocommerce_before_calculate_totals'), 10, 1);
			add_filter('woocommerce_calc_tax', array($this, 'woocommerce_calc_tax'), 1, 1);
			add_filter('woocommerce_get_price', array($this, 'woocommerce_get_price'), 1, 1);
		}

		// Register Widgets
		add_action('widgets_init', array($this, 'register_widgets'));

		// Set the default checkout country to the one selected by the user
		add_filter('default_checkout_country', array($this, 'default_checkout_country'));
		add_filter('get_user_metadata', array($this, 'get_user_metadata'), 10, 4);

		// WC 2.4+
		// Transient keys
		add_filter('woocommerce_get_variation_prices_hash', array($this, 'woocommerce_get_variation_prices_hash'), 10, 3);

		// Add hooks for shortcodes
		$this->set_shortcodes_hooks();
	}

	/**
	 * Returns the country code for the user, detecting it using the IP Address,
	 * if needed.
	 * IMPORTANT: WooCommerce stores the billing country in its "customer" property,
	 * while this method uses WooCommerce session when the billing country is selected.
	 * This must be done because the tax display option is retrieved by WooCommerce
	 * BEFORE the "customer" property is initialised. If we relied on such property,
	 * very often it would be empty, and we would return the incorrect country code.
	 *
	 * @return string
	 */
	public function get_billing_country() {
		if(!empty($this->billing_country)) {
			return $this->billing_country;
		}

		$woocommerce = $this->wc();
		$country = null;

		if(self::doing_ajax() &&
			 (isset($_POST['action']) && ($_POST['action'] === 'woocommerce_update_order_review')) ||
			 // Fix WC 2.4. They silently removed the "action" parameter from the POST
			 // and added a "wc-ajax" in the GET. Yet one more nonsensical change from
			 // the WooCommerce "ninjas"
			 (isset($_REQUEST['wc-ajax']) && ($_REQUEST['wc-ajax'] === 'update_order_review'))) {
			// If user is on checkout page and changes the billing country, get the
			// country code and store it in the session
			check_ajax_referer('update-order-review', 'security');
			if(isset($_POST[Definitions::ARG_BILLING_COUNTRY])) {
				$country = $_POST[Definitions::ARG_BILLING_COUNTRY];
				Aelia_SessionManager::set_value(Definitions::SESSION_BILLING_COUNTRY, $country);
			}
		}

		if(empty($country)) {
			if(isset($_POST[Definitions::ARG_AELIA_BILLING_COUNTRY])) {
				$country = $_POST[Definitions::ARG_AELIA_BILLING_COUNTRY];
				Aelia_SessionManager::set_value(Definitions::SESSION_BILLING_COUNTRY, $country);
			}
		}

		// If no billing country was posted, check if one was stored in the session
		if(empty($country)) {
			$country = Aelia_SessionManager::get_value(Definitions::SESSION_BILLING_COUNTRY);
		}

		// If no valid currency could be retrieved from customer's details, detect
		// it using visitor's IP address
		if(empty($country)) {
			$country = IP2Location::factory()->get_visitor_country();
			Aelia_SessionManager::set_value(Definitions::SESSION_BILLING_COUNTRY, $country);
		}

		// If everything fails, take WooCommerce customer country or base country
		if(empty($country)) {
			if(isset($woocommerce->customer)) {
				$country = $woocommerce->customer->get_country()	;
			}
			else {
				$countries = new \WC_Countries();
				$country = $countries->get_base_country();
			}
		}

		$this->billing_country = $country;
		return $country;
	}

	/**
	 * Indicates if current user has declared to be exempt from tax/VAT.
	 */
	public function get_tax_exempt_status() {
		if(isset($_POST[Definitions::ARG_TAX_DISPLAY_SELECTION])) {
			Aelia_SessionManager::set_value(Definitions::SESSION_TAX_EXEMPT, get_value(Definitions::ARG_TAX_EXEMPT, $_POST, false));
		}
		return Aelia_SessionManager::get_value(Definitions::SESSION_TAX_EXEMPT);
	}

	/**
	 * Returns the tax display setting (including/excluding tax) for current user.
	 *
	 * @param string setting_type Indicates for what price the setting should be
	 * retrieved. Valid values are the following:
	 * - 'sho_prices'
	 * - 'cart_prices'
	 *
	 * @return string "incl" if prices should be displayed including tax, "excl"
	 * otherwise.
	 */
	protected function get_tax_display_setting($prices_type) {
		// If the "tax exempt selector" feature is enabled, and customer declared
		// himself exempt from tax/VAT, display prices excluding tax
		if(self::settings()->get_tax_exempt_enabled() &&
			 $this->get_tax_exempt_status()) {
			return 'excl';
		}

		// Get user country and retrieve the tax settings configured for it
		$user_country = $this->get_billing_country();
		$result = $this->settings_controller()->get_tax_display_for_country($user_country, $prices_type);

		// Debug
		//var_dump($user_country, 'shop', $result);
		return $result;
	}

	/**
	 * Processes the "woocommerce_tax_display_shop" option, eventually replacing
	 * it with the one configured for visitor's country.
	 *
	 * @param string value The original value.
	 * @return string
	 */
	public function option_woocommerce_tax_display_shop($value) {
		$result = $this->get_tax_display_setting('shop_prices');

		return (empty($result)) ? $value : $result;
	}

	/**
	 * Processes the "woocommerce_tax_display_cart" option, eventually replacing
	 * it with the one configured for visitor's country.
	 *
	 * @param string value The original value.
	 * @return string
	 */
	public function option_woocommerce_tax_display_cart($value) {
		$result = $this->get_tax_display_setting('cart_prices');
		return (empty($result)) ? $value : $result;
	}

	/**
	 * Returns the suffix to append to prices.
	 *
	 * @param string default_value The default suffix to display if none is found
	 * for current billing country.
	 * @return string
	 */
	protected function get_tax_suffix($default_value) {
		// If the "tax exempt selector" feature is enabled, and customer declared
		// himself exempt from tax/VAT, display prices excluding tax
		if(self::settings()->get_tax_exempt_enabled() &&
			 $this->get_tax_exempt_status()) {
			$result = self::settings()->get_tax_exempt_price_suffix();
		}
		else {
			$user_country = $this->get_billing_country();
			$result = self::settings()->get_price_suffix_for_country($user_country);
		}

		// Debug
		//var_dump($user_country, $result);
		return (empty($result)) ? $default_value : $result;
	}

	/**
	 * Processes the "woocommerce_price_suffix" option, eventually replacing
	 * it with the one configured for visitor's country.
	 *
	 * @param string value The original value.
	 * @return string
	 */
	public function woocommerce_get_price_suffix($price_suffix, $product) {
		return '<span class="price-suffix">' . $this->get_tax_suffix($price_suffix) . '</span>';
	}

	/**
	 * Processes the "excluding tax or vat" suffix, eventually replacing
	 * it with the one configured for visitor's country.
	 *
	 * @param string price_suffix The original value.
	 * @return string
	 */
	public function woocommerce_countries_ex_tax_or_vat($price_suffix) {
		return $this->get_tax_suffix($price_suffix);
	}

	/**
	 * Processes the "including tax or vat" suffix, eventually replacing
	 * it with the one configured for visitor's country.
	 *
	 * @param string price_suffix The original value.
	 * @return string
	 */
	public function woocommerce_countries_inc_tax_or_vat($price_suffix) {
		return $this->get_tax_suffix($price_suffix);
	}

	/**
	 * Sets customer's country on cart load.
	 */
	public function woocommerce_check_cart_items() {
		// Set customer's country to the pre-selected one, unless a different one was
		// explicitly chosen
		if(!isset($_POST['calc_shipping_country'])) {
			$this->set_customer_country($this->get_billing_country());
			//var_dump($this->get_billing_country());

			do_action('woocommerce_calculated_shipping');
		}
	}

	/**
	 * Determines if one of plugin's admin pages is being rendered. Override it
	 * if plugin implements pages in the Admin section.
	 *
	 * @return bool
	 */
	protected function rendering_plugin_admin_page() {
		$screen = get_current_screen();
		$page_id = $screen->id;

		return ($page_id == 'woocommerce_page_' . Definitions::MENU_SLUG);
	}

	/**
	 * Registers the script and style files needed by the admin pages of the
	 * plugin. Extend in descendant plugins.
	 */
	protected function register_plugin_admin_scripts() {
		// Scripts
		wp_register_script('chosen',
											 '//cdnjs.cloudflare.com/ajax/libs/chosen/1.1.0/chosen.jquery.min.js',
											 array('jquery'),
											 null,
											 true);

		// Styles
		wp_register_style('chosen',
												'//cdnjs.cloudflare.com/ajax/libs/chosen/1.1.0/chosen.min.css',
												array(),
												null,
												'all');
		wp_register_style('jquery-ui',
											'//code.jquery.com/ui/1.10.3/themes/smoothness/jquery-ui.css',
											array(),
											null,
											'all');

		wp_enqueue_style('jquery-ui');
		wp_enqueue_style('chosen');

		wp_enqueue_script('jquery-ui-tabs');
		wp_enqueue_script('jquery-ui-sortable');
		wp_enqueue_script('chosen');

		parent::register_plugin_admin_scripts();
	}

	/**
	 * Loads the scripts required in the Admin section.
	 */
	public function load_admin_scripts() {
		parent::load_admin_scripts();
		$this->localize_admin_scripts();
	}

	/**
	 * Loads the settings that will be used by the Admin scripts.
	 */
	protected function localize_admin_scripts() {
		// Prepare parameters for common admin scripts
		$admin_scripts_params = array(
			'european_union_countries' => $this->wc()->countries->get_european_union_countries(),
		);

		wp_localize_script(static::$plugin_slug . '-admin',
											 'aelia_tdbc_params',
											 $admin_scripts_params);
	}

	/**
	 * Loads Styles and JavaScript for the frontend. Extend as needed in
	 * descendant classes.
	 */
	public function load_frontend_scripts() {
		// Enqueue the required Frontend stylesheets
		wp_enqueue_style(static::$plugin_slug . '-frontend');

		// JavaScript
		wp_enqueue_script(static::$plugin_slug . '-frontend');
	}

	/**
	 * Returns the full path and file name of the specified template, if such file
	 * exists.
	 *
	 * @param string template_name The name of the template.
	 * @return string
	 */
	public function get_template_file($template_name) {
		$template = '';

		/* Look for the following:
		 * - yourtheme/woocommerce-aelia-currencyswitcher-{template_name}.php
		 * - yourtheme/woocommerce-aelia-currencyswitcher/{template_name}.php
		 */
		$template = locate_template(array(
			self::$plugin_slug . "-{$template_name}.php",
			self::$plugin_slug . '/' . "{$template_name}.php"
		));

		// If template could not be found, get default one
		if(empty($template)) {
			$default_template_file = $this->path('views') . '/' . "{$template_name}.php";

			if(file_exists($default_template_file)) {
				$template = $default_template_file;
			}
		}

		// If template does not exist, trigger a warning to inform the site administrator
		if(empty($template)) {
			$this->trigger_error(Definitions::INVALID_TEMPLATE,
													 E_USER_WARNING,
													 array(self::$plugin_slug, $template_name));
		}

		return $template;
	}

	/**
	 * Registers all the Widgets used by the plugin.
	 */
	public function register_widgets() {
		$this->register_widget('Aelia\WC\TaxDisplayByCountry\Billing_Country_Selector_Widget');
	}

	/**
	 * Returns the default checkout country.
	 *
	 * @param string checkout_country The country passed by WooCommerce.
	 * @return string
	 */
	public function default_checkout_country($checkout_country) {
		return $this->get_billing_country();
	}

	/**
	 * Intercepts the fetching of user metadata, to alter the billing address if
	 * needed.
	 *
	 * @param mixed value The original value of the user metadata.
	 * @param int user_id The user for whom the data is being retrieved.
	 * @param string meta_key Optional. Metadata key. If not specified, retrieve
	 * all metadata for the specified object.
	 * @param bool $single Optional, default is false. If true, return only the
	 * first value of the specified meta_key. This parameter has no effect if
	 * meta_key is not specified.
	 * @return string
	 */
	public function get_user_metadata($value, $user_id, $meta_key, $single) {
		// If we are on checkout page and the billing country is requested for
		// current user, retrieve the one he (eventually) selected
		if(($meta_key === 'billing_country') &&
			 (defined('WOOCOMMERCE_CHECKOUT') || is_checkout()) &&
			 ($user_id === wp_get_current_user()->ID)) {
			return $this->get_billing_country();
		}

		return $value;
	}

	/**
	 * Alters the transient key to retrieve the prices of variable products,
	 * ensuring that the currency is taken into account.
	 *
	 * @param array cache_key_args The arguments that form the cache key.
	 * @param WC_Product product The product for which the key is being generated.
	 * @param bool display Indicates if the prices are being retrieved for display
	 * purposes.
	 * @return array
	 * @since WooCommerce 2.4+
	 * @since 1.7.10.150914
	 */
	public function woocommerce_get_variation_prices_hash($cache_key_args, $product, $display) {
		// Debug
		//$cache_key = 'wc_var_prices' . md5(json_encode($cache_key_args));
		//delete_transient($cache_key);

		$cache_key_args[] = get_option('woocommerce_tax_display_shop');
		$cache_key_args[] = get_option('woocommerce_tax_display_cart');
		return $cache_key_args;
	}

	/**
	 * Triggered before cart totals are calculated.
	 * If the option is enabled, it overrides shops' base location in order to allow
	 * keeping product prices fixed, regardless of what VAT rate applies to the
	 * customer.
	 *
	 * @param WC_Cart cart The cart object.
	 * @since 1.7.0.150109
	 */
	public function woocommerce_before_calculate_totals($cart) {
		if(self::settings()->keep_prices_fixed()) {
			$this->set_shop_base_location_overrides(true);
		}
	}

	/**
	 * Triggered before a product price is retrieved.
	 * This method is just used as an event to override shop's base location for
	 * the "fixed product prices" feature.
	 *
	 * @param float price The price passed by WooCommerce.
	 * @return float The price received, unaltered.
	 * @since 1.7.0.150109
	 */
	public function woocommerce_get_price($price) {
		if(self::settings()->keep_prices_fixed()) {
			$this->set_shop_base_location_overrides(true);
		}
		return $price;
	}

	/**
	 * Triggered after the tax calculation is completed.
	 * This method removes the overrides for shop's base location
	 *
	 * @param array taxes An array of taxes.
	 * @return array The array of taxes, unaltered.
	 * @since 1.7.0.150109
	 */
	public function woocommerce_calc_tax($taxes) {
		$this->set_shop_base_location_overrides(false);
		return $taxes;
	}

	/**
	 * Sets or unsets the filters that will override shop's base location.
	 *
	 * @param bool enable_overrides Indicates if the filters should be set or unset.
	 * @since 1.7.0.150109
	 */
	protected function set_shop_base_location_overrides($enable_overrides) {
		global $woocommerce;
		if(empty($woocommerce->customer)) {
			$this->log(__('Customer object is empty, base location override skipped.', self::$text_domain));
			return;
		}

		$hooks = array(
			'woocommerce_countries_base_country' => 'override_base_country',
			'woocommerce_countries_base_state' => 'override_base_state',
			'woocommerce_countries_base_city' => 'override_base_city',
			'woocommerce_countries_base_postcode' => 'override_base_postcode',
		);

		/* Enable or disable the overrides, depending on the flag. This method could
		 * be written using call_user_func(), instead of using two identical loops.
		 * The loops have been chosen because they are easier to read and to understand.
		 */
		if($enable_overrides) {
			foreach($hooks as $hook => $filter) {
				add_filter($hook, array($this, $filter), 10, 1);
			}
		}
		else {
			foreach($hooks as $hook => $filter) {
				remove_filter($hook, array($this, $filter), 10, 1);
			}
		}
	}

	/**
	 * Returns an array of elements containing customer's taxable address.
	 *
	 * @return array An array with the elements forming customer's address (country,
	 * state, city and postcode).
	 * @since 1.7.7.150831
	 */
	protected function get_customer_taxable_address() {
		global $woocommerce;
		return $woocommerce->customer->get_taxable_address();
	}

	/**
	 * Overrides shop's base country using the corresponding information from
	 * customer's taxable address.
	 *
	 * @param string country The original base country.
	 * @return string The country, from customer's taxable address.
	 * @since 1.7.0.150109
	 */
	public function override_base_country($country) {
		// WC 2.4+
		// Prevent infinite recursion by using a semaphore
		static $processing = false;
		if($processing) {
			return $country;
		}
		$processing = true;

		$customer_taxable_address = $this->get_customer_taxable_address();
		// Customer's country is the first element in the returned array
		$country = get_value(0, $customer_taxable_address);

		$processing = false;
		return $country;
	}

	/**
	 * Overrides shop's base state using the corresponding information from
	 * customer's taxable address.
	 *
	 * @param string state The original base state.
	 * @return string The state, from customer's taxable address.
	 * @since 1.7.0.150109
	 */
	public function override_base_state($state) {
		// WC 2.4+
		// Prevent infinite recursion by using a semaphore
		static $processing = false;
		if($processing) {
			return $state;
		}
		$processing = true;

		$customer_taxable_address = $this->get_customer_taxable_address();
		// Customer's state is the second element in the returned array
		$state = get_value(1, $customer_taxable_address);

		$processing = false;
		return $state;
	}

	/**
	 * Overrides shop's base city using the corresponding information from
	 * customer's taxable address.
	 *
	 * @param string city The original base city.
	 * @return string The city, from customer's taxable address.
	 * @since 1.7.0.150109
	 */
	public function override_base_city($city) {
		// WC 2.4+
		// Prevent infinite recursion by using a semaphore
		static $processing = false;
		if($processing) {
			return $city;
		}
		$processing = true;

		$customer_taxable_address = $this->get_customer_taxable_address();
		// Customer's city is the third element in the returned array
		$city = get_value(2, $customer_taxable_address);

		$processing = false;
		return $city;
	}

	/**
	 * Overrides shop's base postcode using the corresponding information from
	 * customer's taxable address.
	 *
	 * @param string postcode The original base postcode.
	 * @return string The postcode, from customer's taxable address.
	 * @since 1.7.0.150109
	 */
	public function override_base_postcode($postcode) {
		// WC 2.4+
		// Prevent infinite recursion by using a semaphore
		static $processing = false;
		if($processing) {
			return $postcode;
		}
		$processing = true;

		$customer_taxable_address = $this->get_customer_taxable_address();
		// Customer's postcode is the fourth element in the returned array
		$postcode = get_value(3, $customer_taxable_address);

		$processing = false;
		return $postcode;
	}

	/**
	 * Sets hooks to register shortcodes.
	 */
	protected function set_shortcodes_hooks() {
		// Shortcode to render the billing country selector
		add_shortcode('aelia_tdbc_billing_country_selector_widget',
									array('Aelia\WC\TaxDisplayByCountry\Billing_Country_Selector_Widget', 'render_billing_country_selector'));
	}

	/**
	 * Registers a widget class.
	 *
	 * @param string widget_class The class to register.
	 * @param bool stop_on_error Indicates if the function should raise an error
	 * if the Widget Class doesn't exist or cannot be loaded.
	 * @return bool True, if the Widget was registered correctly, False otherwise.
	 */
	protected function register_widget($widget_class, $stop_on_error = true) {
		register_widget($widget_class);
		return true;
	}
}

$GLOBALS[WC_Aelia_Tax_Display_By_Country::$plugin_slug] = WC_Aelia_Tax_Display_By_Country::factory();
